#!/bin/bash

#########################################################
#														#
#	Bash script to properly install LABE				#
#														#
#	Configuration files modified:						#
#		-/etc/openldap/slapd.conf						#
#		-/etc/openldap/schema/core.schema				#
#		-/etc/openldap/schema/cosine.schema				#
#		-/etc/php.ini									#
#														#
#	Author: Mathieu Leduc-Hamel							#
#		Savoir-Faire Linux								#
#														#
#########################################################

LB="\033[1;34m"
WC="\033[1;37m"
RC="\033[1;31m"
NC="\033[0m"
ERROR=0

echo -e "${LB}###################################"
echo "#   Welcome to LABE setup script  #" 
echo "#                                 #"				     
echo "# This script will modify some    #"
echo "# configuration files on your     #"
echo "# system and save your original   #"
echo "# files with the extension '.orig'#"
echo "#                                 #"
echo "# Hope you'll like this software  #"
echo "#                                 #"
echo -e "###################################${NC}"

echo -en "${LB}Are you sure you want to install LABE of your system? <y/n>:${NC} "
read -n1 VAR
echo ""

if [ $VAR == "y" ]; then
	#########################################################
	### /etc/openldap/slapd.conf ############################
	#########################################################
	echo -e "${WC}Checking for slapd.conf in /etc/openldap/...${NC}"
	if [ -e /etc/openldap/slapd.conf ]; then

		echo -e "${WC}Checking if extension.schema ispresent in slapd.conf${NC}"

		grep "/etc/openldap/schema/extension.schema" /etc/openldap/slapd.conf > /dev/null

		if [ $? -eq 1 ]; then
			echo "     The schema extension.schema is not present in slapd.conf, adding it"

			awk -v file=$(grep include /etc/openldap/slapd.conf | tail -1 | awk '{ print $2 }' ) '
				{ print }
				$1 == "include" && $2 == file {
					print "include         /etc/openldap/schema/extension.schema"
				}
			' "/etc/openldap/slapd.conf" > /etc/openldap/slapd.temp
		
			echo "     Saving /etc/openldap/slapd.conf as /etc/openldap/slapd.conf.orig"
			cp -a /etc/openldap/slapd.conf /etc/openldap/slapd.conf.orig

			cp /etc/openldap/slapd.temp /etc/openldap/slapd.conf
			rm /etc/openldap/slapd.temp
		else
			echo "     No modification to do"
		fi
	else
		echo -e "     ${RC}Cannot find slapd.conf in /etc/openldap${NC}"
		ERROR=1
	fi
	### End of slapd.conf modification ######################


	#########################################################
	### /etc/openldap/schema/core.schema ####################
	#########################################################
	echo -e "${WC}Checking for core.schema in /etc/openldap/schema...${NC}"
	if [ -e /etc/openldap/schema/core.schema ]; then

		echo -e "${WC}Checking for the 'telephoneNumber' attribute in /etc/openldap/schema/core.schema${NC}"

		grep "attributetype ( 2.5.4.20 NAME 'telephoneNumber'" /etc/openldap/schema/core.schema > /dev/null
		
		if [ $? -eq 0 ]; then
		echo "     Attribute found, adding a new synonyme for it"
		echo "     Saving /etc/openldap/schema/core.schema as /etc/openldap/schema/core.schema.orig"
		cp -a /etc/openldap/schema/core.schema /etc/openldap/schema/cosine.schema.orig
 	
                echo "$(sed -e 's/^.*2\.5\.4\.20.*$/attributetype ( 2.5.4.20 NAME ('\''telephoneNumber'\'' '\''xmozillaanyphone'\'')/' /etc/openldap/schema/core.schema)" > /etc/openldap/schema/core.schema
		else
			echo "     No modification to do"
		fi
	else
		echo -e "     ${RC}Cannot find the ldap schema: core.schema${NC}"
		ERROR=1
	fi
	### end of core.schema modification #####################


	#########################################################
	### /etc/openldap/schema/cosine.schema ##################
	#########################################################
	echo -e "${WC}Checking for cosine.schema in /etc/openldap/schema...${NC}"
	if [ -e /etc/openldap/schema/cosine.schema ]; then

		echo -e "${WC}Checking for the 'mobile' and 'mobileTelephoneNumber' attributes in /etc/openldap/schema/cosine.schema...${NC}"

		grep "NAME ( 'mobile' 'mobileTelephoneNumber' )" /etc/openldap/schema/cosine.schema  > /dev/null

		if [ $? -eq 0 ]; then
			echo -e "     ${WC}Attributes found, adding a new synonyme for them${NC}"
			echo -e "     ${WC}Saving /etc/openldap/schema/cosine.schema as /etc/openldap/schema/cosine.schema.orig${NC}"
			cp -a /etc/openldap/schema/cosine.schema /etc/openldap/schema/cosine.schema.orig  

			echo "$(sed -e 's/NAME ( '\''mobile'\'' '\''mobileTelephoneNumber'\'' )/NAME ( '\''mobile'\'' '\''mobileTelephoneNumber'\'' '\''cellphone'\'' )/' /etc/openldap/schema/cosine.schema)" > /etc/openldap/schema/cosine.schema
		else
			echo "     No modification to do"
		fi

		echo -e "${WC}Checking for the 'pager' and 'pagerTelephoneNumber' attributes in /etc/openldap/schema/cosine.schema...${NC}"

		grep "'pager' 'pagerTelephoneNumber' )" /etc/openldap/schema/cosine.schema > /dev/null

		if [ $? -eq 0 ]; then
			echo "     Attributes found, adding a new synonyme for them"

			if [ -e /etc/openldap/schema/cosine.schema.orig ]; then
				echo "     /etc/openldap/schema/cosine.schema as been already saved"
			else
				echo "     Saving /etc/openldap/schema/cosine.schema as /etc/openldap/schema/cosine.schema.orig"
				cp -a /etc/openldap/schema/cosine.schema /etc/openldap/schema/cosine.schema.orig
			fi

			echo "$(sed -e 's/NAME ( '\''pager'\'' '\''pagerTelephoneNumber'\'' )/NAME ( '\''pager'\'' '\''pagerTelephoneNumber'\'' '\''pagerPhone'\'' )/' /etc/openldap/schema/cosine.schema)" > /etc/openldap/schema/cosine.schema
		else
			echo "     No modification to do"
		fi
	else
		echo -e "     ${RC}Cannot find the ldap schema: cosine.schema${NC}"
		ERROR=1
	fi
	### end of cosine.schema modification ###################


	#########################################################	
	### Check for the basedn ################################
	#########################################################
	echo -e "${WC}Checking for the ldap root directory...${NC}"

	PORT=389
	HOST=127.0.0.1
	SUFFIX="dc=example,dc=com"
	MANAGER=manager

	echo "     Please enter... (take a look at /etc/openldap/slapd.conf)"

	echo -n "          the address of you ldap server [127.0.0.1]: "
	read TEMP_HOST
	if [[ -n "$TEMP_HOST" ]]; then
	    HOST="$TEMP_HOST"
	fi
	
	echo -n "          the port of your ldap server [389]: "
	read TEMP_PORT
	if [[ -n "$TEMP_PORT" ]]; then 
		PORT="$TEMP_PORT"
	fi
	
	echo -n "          the basedn for labe [dc=example,dc=com]: "
	read TEMP_DN
	if [[ -n "$TEMP_DN" ]]; then
		SUFFIX="$TEMP_DN"
	fi

	echo -n "          the manager of this directory [manager]: "
	read TEMP_MANAGER
	if [[ -n "$TEMP_MANAGER" ]]; then
		MANAGER="$TEMP_MANAGER"
	fi
	
	until [[  -n $PASSWORD ]]; do
		echo -n "          the password of this manager: "
		read -s PASSWORD
		echo ""
	done

	ROOTDN="cn=${MANAGER},${SUFFIX}"
              
	ldapsearch -x -h $HOST -p $PORT -b $SUFFIX '(objectclass=*)' > /dev/null

	if [[ $? -eq 0 ||  $? -eq 32 ]]; then
		echo "     Adding the LDAP root entry"
		ldapadd -x -h 127.0.0.1 -p $PORT -D $ROOTDN -w $PASSWORD << EOF > /dev/null &> /dev/null
dn: $SUFFIX
objectclass: dcObject
objectclass: organization
o: LDAP First entry 
dc: example

dn: $ROOTDN
objectclass: organizationalRole
cn: $MANAGER
EOF
		RESULT=$?
		if [ $RESULT -eq 0 ]; then
			echo "     LDAP root entry has been add successfuly!"
		elif [ $RESULT -eq 68 ]; then
			echo -e "          ${RC}LDAP root entry was already created!${NC}" 
			ERROR=1
		else
			echo -e "          ${RC}Could not add the LDAP root entry!${NC}"
			ERROR=1
		fi
	fi

	### Writing the informations in connect.conf ############
	#if  [ $RESULT -eq 0 ]; then
	    echo -e "     ${WC}Do you want the LDAP information in your"
	    echo -en "     LABE configuration file? <y/n>:${NC} "
	    read -n1 VAR
	    echo ""

	    if [ $VAR == "y" ]; then
		PASSWORD=$(echo -n "secret"|md5sum|  sed 's/ -//')
		cat << EOF > /etc/labe/connect.conf
;;
;; LABE Configuration
;;
;; Syntax:
;;  1. Identifier
;;  2. Address
;;  3. Port
;;  4. Bind
;;  5. md5sum of the password (man md5sum for more info)
;;  6. Rootdn
;;

;;
;; Main server
;;
main
${HOST}
${PORT}
${SUFFIX}
${PASSWORD}
${ROOTDN}
EOF
		echo "          LABE configuration file successfuly created!"

	    fi
	#fi

	### Restarting ldap server ##############################
	if [ -x /etc/init.d/ldap ]; then
        	echo -e "${WC}Restarting slapd deamon...${NC}"
                /etc/init.d/ldap restart > /dev/null &> /dev/null
		echo "     The slapd deamon has been restart"
        fi


	#########################################################
	### /etc/php.ini ########################################
	### Make sure that the web server accept http upload ####
	#########################################################
	echo -e "${WC}Checking for php.ini in /etc...${NC}"
	if [ -e /etc/php.ini ]; then
		echo "     /etc/php.ini found!"	
		echo -e "${WC}Checking file_uploads in /etc/php.ini...${NC}"

        	grep "file_uploads = Off" /etc/php.ini > /dev/null

        	if [ $? -eq 0 ]; then
                	echo "    file_uploads is Off, changing it to On"
                	echo "    Saving /etc/php.ini as /etc/php.ini.orig"
                	cp -a /etc/php.ini /etc/php.ini.orig

                	echo "$(sed -e 's/file_uploads = Off/file_uploads = On/' /etc/php.ini)" > /etc/php.ini

                	if [ -x /etc/init.d/httpd ]; then
				echo "Restarting the web server..."
                        	/etc/init.d/httpd restart > /dev/null
				echo "    The web server has been restarted"
			else
				echo "    You'll have to restart your web server"
                	fi
		else
			echo "     No modification to do"
        	fi

	else
		echo -e "    ${RC}/etc/php.ini not found${NC}"
		ERROR=1
	fi
	### end of upload check ################################


	echo ""
	if (( $ERROR > 0 )); then
		echo -e "${RC}Some error were reported, maybe LABE will not work... check the INSTALL file${NC}"	
	else
		echo -e "${LB}LABE is now supposed to fully fonctional, Have fun!${NC}"
	fi
else
	echo -e "${RC}LABE is not properly installed, check the INSTALL file${NC}"
fi

echo ""
echo "Please report any bugs and comments to <labe@savoirfairelinux.com>" 
