<?php

    /**
     * LABE-The LDAP Addresse Book Editor
     */

    /**
     * Super class of labe_main
     */
    require(dirname(__FILE__).'/'.'labe_util.class.php');

    /**
     * Class of the implementation of the object comparison
     */
    require(dirname(__FILE__).'/'.'labe_cmpObject.class.php');

    /** 
     *  Implement all necessary to connect to a ldap server and to retrieve, and add modified information from it.
     *  @authors Mathieu Leduc-Hamel
     *  @copyright Savoir-faire Linux inc.
     */
    class labe_main extends labe_util {
	/**
	 * Variable indicating the name of the ldap host
	 * @var string
	 */
	var $name;

	/**
	 * Variable indicating the address of the ldap host
	 * @var string
	 */
	var $host;

	/**
	 * Variable indicating the port of the ldap host
	 * @var string
	 */
	var $port;
	
	/**
	 * Variable indicating the bind dn (distinguish name) of the ldap connexion
	 * @var string
	 */
	var $bind;
	
	/**
	 * Variable indicating the base ldap root dn of the ldap connexion
	 * @var string
	 */
	var $rootdn;

	/**
	 * Variable indicating the password for the bind dn of the ldpa connexion
	 * @var string
	 */
	var $passwd;
		
	/**
	 * Flag indicating if all connexion information are there
	 * @var integer
	 */
	var $FLAG_INFO = 0;

	/**
	 * Variable containing the name of the configuration file
	 * @var string
	 */
	var $OPTION_FILE;

	/**
	 * Variable containing data to be delete of a selected contact
	 * @var array
	 */
	var $delval;

	/**
	 * Variable containing the name of allowed attributes
	 * @var array
	 */
	var $KeyEntry;

	var $util;


	/**
	 * Called on the construction of an instance of this class.
	 * It read the option file and set all information about the connexion.
	 *
	 * @param string $optfile the path to the connexion parameters file.
	 *
	 * @return none
	 */
	function labe_main($optFile) {
	    // Set options of the connexion
	    $this->OPTION_FILE = $optFile;
	    $this->delval = array();
	    $this->encoding = 1;

	    // Table of attributes
	    $this->KeyEntry = array(
		"dn",
		"sn",
		"givenname",
		"initials",
		"mail",
		"jpegphoto",
		"homephone",
		"homepostaladdress",
		"mobile",
		"o",
		"postaladdress",
		"l",
		"st",
		"postalcode",
		"c",
		"title",
		"ou",
		"physicaldeliveryofficename",
		"telephonenumber",
		"facsimiletelephonenumber",
		"url",
		"comment",
		"conferenceinformation",
		"usercertificate",
		"reports",
		"otherfacsimiletelephonenumber",
		"pager",
		"ipphone",
		"erase");
	}


	/**
	 * Connect to a ldap server and return the status of the operation
	 *
	 * @param string $nHost the address of the host
	 * @param string $nPort the port of the host
	 *
	 * @return On success it return the connexion link if not success, false
	 */
	function Connect_LDAP($nHost, $nPort) {
	    $connexion = @ldap_connect($nHost, $nPort);
	    @ldap_set_option($connexion, LDAP_OPT_PROTOCOL_VERSION, 3);

	    if(!$this->passwd) {
		$this->passwd = crypt(microtime());
	    }

	    $bind_result = @ldap_bind($connexion,$this->rootdn,$this->passwd);
	    if($bind_result != FALSE) {
		return $connexion;
	    }
	    else {
		return FALSE;
	    }
	}


	/**
	 * Close the connexion with a ldap server and return the close status
	 * 
	 * @param string $ldap a valid ldap directory connexion link
	 *
	 * @return on succuss true else false
	 */
	function Close_LDAP($ldap) {
	    if(@ldap_close($ldap)) {
		return TRUE;
	    }
	    else {
		return FALSE;
	    }
	}


	/**
	 * Read the option file and set all correct values
	 *
	 * @param string $name The name of the server
	 *
	 * @return On success true else false
	 */
	function getInfo($name) {
	    // Reading information in the option's file
	    $temp = $this->parse_cfg($this->OPTION_FILE);
	    
	    for($i = 0; $i < count($temp); $i++)
	    {
		if(strcmp($name,$temp[$i]["name"]) == 0) {
		    $this->name = $temp[$i]["name"];
		    $this->host = $temp[$i]["host"];
		    $this->port = $temp[$i]["port"];
		    $this->bind = $temp[$i]["bind"];
		    $this->rootdn = $temp[$i]["rootdn"];
		    $this->charset = $temp[$i]["charset"];
		    return $temp[$i];
		}
	    }
	    return false;
	}
	

	/**
	 * Set the password of the labe object
	 *
	 * @param string $password The new password to set
	 *
	 * @return none
	 */
	function setPass($password) {
	    $this->passwd = $password;
	}


	/**
	 * Add contact to the ldap directory of the main host.
	 *
	 * it call to the function "generateDN" to create a new DN for the futur contact.
	 *
	 * @param string  $connexion The connexion link with the ldap directory
	 * @param array $entry The information about the contact to be added to the directory.
	 *
	 * @return On success true else false
	 */
	function AddContact($connexion,$entry) {
	    if(@ldap_add($connexion,$this->generateDN($connexion,$entry["sn"],$entry["givenname"]),$entry)) {
		    return TRUE;
	    }
	    else {
		return FALSE;
	    }
	}


	/**
	 * Remove an existing contact in ldap directory
	 *
	 * @param string  $connexion The connexion link with the ldap directory
	 * @param array $ids The array of contact to be remove
	 *
	 * @return none
	 */
	function RemoveContact($connexion,$ids) {
	    $nb = count($ids);
	    while($nb > 0) {
		$ToDel= ereg_replace("_"," ",$ids[$nb-1]);
		if(!ldap_delete($connexion,$ToDel))
		    echo "Cannot delete the contact!: $ToDel";	
		$nb--;
	    }
	}


	/**
	 * Modify the value of attributes or remove attributes, for one contact
	 *
	 * @param string $connexion The connexion link with the ldap directory
	 * @param string $dn DN of the contact to modify
	 * @param array $entry The value of his attributes
	 *
	 * @return On success true else false
	 */
	function ModifyContact($connexion,$dn,$entry) {
	    // Modifying value
	    @ldap_modify($connexion,$dn,$entry);

	    $entry2 = @ldap_get_entries($connexion,ldap_search($connexion,$dn,"(objectclass=officePerson)"));

	    // Organizing an entry for the ldap_mod_del
	    for($i = 0; $i < count($this->delval) ; $i++) {
		if($this->delval[$i] != "givenname" && $this->delval[$i] != "sn") { 
		    $totl = $entry2[0][$this->delval[$i]]["count"];
		    for($l = 0; $l < $totl; $l++) {
			if($totl == 1) {
			    $findel[$this->delval[$i]] = array();
			}
			elseif($totl > 1){
			    $findel[$this->delval[$i]][$l] = $entry2[0][$this->delval[$i]][$l];
			}
		    }
		}				
	    }
	    // Deleting empty attributes
	    if($findel) {
		if(!@ldap_mod_del($connexion,$dn,$findel)) {
		    echo "Cannot delete an attribute";
		    return FALSE;
		}
		return TRUE;
	    }
	    else {
		return FALSE;
	    }
	}


	/**
	 * Create an array of attributes for an entry in a ldap directory.
	 *
	 * Create another array of the attributes to be remove (for the modification - delval - )
	 *
	 * @param boolean $mod The flag indicating that we are in modification
	 * @param array $VARS_ARRAY List of all attributes that we can modify
	 *
	 * @return On success an array of all the attributes, else false
	 */
	function createEntry($mod = TRUE, $VARS_ARRAY) {
			
	    $this->delval = array();

	    global $HTTP_POST_FILES;
	    $entry["objectclass"]="officePerson";

	    // Checking if attributes are there, if not, put it in the delval array
	    if($VARS_ARRAY["sn"] && $VARS_ARRAY["givenname"]) {
		$entry["cn"] =  $this->encode($VARS_ARRAY["sn"]." ".$VARS_ARRAY["givenname"]);
	    }

	    for($i = 1; $i < count($this->KeyEntry) -1; $i++ ) {
		if(!empty($VARS_ARRAY[$this->KeyEntry[$i]])) {
		    $entry[$this->KeyEntry[$i]] = $this->encode(stripslashes($VARS_ARRAY[$this->KeyEntry[$i]]));
		}
		else {
		    if($this->KeyEntry[$i] != "jpegphoto" && $this->KeyEntry[$i] != "sn" && $this->KeyEntry[$i] != "givenname")
			$this->delval[] = $this->KeyEntry[$i];
		}
	    }
		
	    if(is_uploaded_file($HTTP_POST_FILES['jpegphoto']['tmp_name'])) {
		$filename = "/var/tmp/labe_file".date("his");
		
		move_uploaded_file($HTTP_POST_FILES['jpegphoto']['tmp_name'],$filename);
		
		$file=fopen($filename,"r");	
		$temp=fread($file,filesize($filename));
		
		fclose($file);
		
		$codephoto=base64_encode($temp);
		$entry["jpegphoto"] = $codephoto;
	    }
	    elseif($VARS_ARRAY["erase"]) {
		$this->delval[] = "jpegphoto";
	    }

	    return $entry;
	}


	/**
	 * Create a DN with a name and a first name. If the already exist, it add a number after the name
	 *
	 * @param string $connexion The connexion link with the ldap directory
	 * @param string $fistname The firstname of the contact
	 * @param string $lastname The last name of the contact
	 *
	 * @return The new DN of the contact
	 */
	function generateDN($connexion,$firstname,$lastname) {
	    $i = 0;
	    $result = TRUE;

	    do {
		if($i > 0)
		    $dn = "cn= ".$firstname." ".$lastname." ".$i." ,".$this->bind; 
		else
		    $dn = "cn= ".$firstname." ".$lastname." ,".$this->bind;

		if($this->searchContact($connexion,"(objectclass=officePerson)",$dn) != FALSE)
		    $i += 1;
		else
		    $result = FALSE;
		
	    } while($result);

	    if($i != 0)
		return("cn= ".$firstname." ".$lastname." ".$i." ,".$this->bind);
            else
		return("cn= ".$firstname." ".$lastname." ,".$this->bind);
	}


	/**
	 * Search for all the contact in ldap.
	 *
	 * @param string $connexion The connexion link with the ldap directory
	 * @param string $filter The search filter
	 *
	 * @return An array containing the search result
	 */
	function SearchALL($connexion,$filter = "(objectclass=officePerson)") {
	    $result = @ldap_search($connexion, $this->bind, $this->encode($filter));
	    $entries = @ldap_get_entries($connexion, $result);

	    return $this->imptUTF8($entries);
	}
	

	/**
	 * Search for particular contact
	 * 
	 * @param string $connexion The connexion link with the ldap directory
	 * @param string $filter The search filter
	 * @param string $dn The dn of the particular contact
	 *
	 * @return On success, an array containing the search result
	 */
        function SearchContact($connexion,$filter,$dn = FALSE) {
	    if(!$dn)$dn = $this->bind;	
		
	    $dn = ereg_replace("_"," ",$dn);

	    $result = @ldap_search($connexion, $dn, $this->encode($filter));
		
	    if($result) {
		$entries = @ldap_get_entries($connexion, $result);
	    }
	    else {
		$entries = FALSE;
	    }

	    return $this->imptUTF8($entries);
	}

	
	/**
	 * Sort contact by a defined attribute
	 *
	 * @param string $connexion The connexion link with the ldap directory
	 * @param array $entry The array to sort
	 * @param string $filter The sort parameter
	 *
	 * @return The sorted array
	 */
	function SortEntries($connexion,$entry,$filter) {
	    setlocale(LC_CTYPE,"de");
	    $cmp = new CmpObject($filter);
	    @uasort($entry,array($cmp, 'cmp'));

	    $idx = 0;
	    $result = array();

	    @reset($entry);

	    while($i=@current($entry)) {
		$key = key($entry);
		if(is_array($i)) {
		    $key = $idx;
		    ++$idx;
		}
		$result[$key] = $i;
		next($entry);
	    }
	    return $result; 
	}
    }
?>
