<?php
    /**
     * LABE - The LDAP Address Book Editor
     */

    /**
     * Bunch of utilities used in labe
     * @author  Mathieu Leduc-Hamel
     * @copyright  Savoir-faire Linux inc.
     */
    class labe_util {
	/**
	 * Variable containing the character encoding used by the LABE (i.e. ISO-8859-1)
	 * @var integer
	 */
	var $encoding;
	
	/**
	 * Remove special caracters of a string
	 *
	 * @param string $string The string where we want to take out special caracters
	 *
	 * @return The string without accent
	 */
	function removeaccents($string) { 
	    return strtr($string, "???????", "SOZsozYYuAAAAAAACEEEEIIIIDNOOOOOOUUUUYsaaaaaaaceeeeiiiionoooooouuuuyy"); 
	}


	/**
	 * Decode a unicode (UTF-8) string to the standard encoding (iso-8859-1)
	 *
	 * @param array $entry The array of data to decode
	 *
	 * @return The array decoded
	 */
        function imptUTF8($entry) {
            for($i = 0; $i < count($entry)-1; $i++) {
                for($j = 0; $j <count($this->KeyEntry); $j++) {
                    $entry[$i][$this->KeyEntry[$j]][0] = htmlspecialchars(
			$this->decode($entry[$i][$this->KeyEntry[$j]][0])
		    );
                }
            }
            return $entry;
	}


	/**
	 * Encode in another character encoding
	 *
	 * @param string $string The string to encode in a different encoding
	 *
	 * @return The string encoded in a different encoding
	 */
	 function encode($string) {
		return iconv($this->charset,"UTF-8",$string);
	 }


	/**
	 * Decode in another character encoding
	 *
	 * @param string $string The string to decode in a different encoding
	 *
	 * @return The string encoded in a different encoding
	 */
	function decode($string) {
		return iconv("UTF-8",$this->charset,$string);
	}
       
       
	/**
	 *  Parse a ldif file and return an valid entries array for labe
	 *
	 *  @param string $file The file ressource to parse
	 *
	 *  @return On success the entries array
	 */
	function parse_ldif($file) {

	    if($file) {
		$nb = -1;
		$entry;
	    
		$buffer = trim(fgets($file, 4096));
		while (!feof($file)) {
		    $pos = strpos($buffer, ":");
		    if(!($pos === false)) {
			$arg1 = substr($buffer,0,$pos);	
			if(strnatcasecmp($arg1,"dn") == 0) {
			    $nb++;
			}
			if(substr($buffer,$pos+1,strlen($buffer))) {
			    $arg2 = substr($buffer,$pos+1,strlen($buffer));
			    $entry[$nb][$arg1] = $this->encode($arg2);
			}
		    }
		    $buffer = trim(fgets($file, 4096));
		}
		fclose($file);
		return $entry;
	    }
	    else {
		return false;
	    }
	}


	/**
	 * Parse a config file and return his content
	 *
	 * @param string The file name of the file
	 *
	 * @return On success the array of value of this file, on failure, false.
	 */
	function parse_cfg($filename) {
	    $file = @fopen($filename,"r");

	    if($file) {
		$attrib = array("name", "host", "port", "bind", "passwd", "rootdn","charset");
		$i = 0;
	    
		$temp = ereg_replace(" ", "",trim(fgets($file,255)));

		while(!feof($file)) {
		    for($l = 0; $l < count($attrib); $l++) {
			while(!(strpos($temp,";") === false || strpos($temp,";") > 1)  || empty($temp) && !feof($file)) {
			    $temp = ereg_replace(" ", "",trim(fgets($file,255)));
			}
			$entry[$i][$attrib[$l]] = $temp;
			$temp = (trim(fgets($file,255)));
		    }
		    $i++;
		}
		return $entry;
	    }
	    else {
		return false;
	    }
	}
}
